$path = $MyInvocation.MyCommand.Path
$src = (Split-Path -Parent -Path $path) -ireplace '[\\/]tests[\\/]unit', '/src/'
Get-ChildItem "$($src)/*.ps1" -Recurse | Resolve-Path | ForEach-Object { . $_ }

$PodeContext = @{ 'Server' = $null; }

Describe 'Find-PodeRoute' {
    Context 'Invalid parameters supplied' {
        It 'Throw invalid method error for no method' {
            { Find-PodeRoute -Method 'MOO' -Path '/' } | Should Throw "Cannot validate argument on parameter 'Method'"
        }

        It 'Throw null route parameter error' {
            { Find-PodeRoute -Method GET -Path $null } | Should Throw 'The argument is null or empty'
        }

        It 'Throw empty route parameter error' {
            { Find-PodeRoute -Method GET -Path ([string]::Empty) } | Should Throw 'The argument is null or empty'
        }
    }

    Context 'Valid method and route' {
        It 'Return null as method does not exist' {
            $PodeContext.Server = @{ 'Routes' = @{}; }
            Find-PodeRoute -Method GET -Path '/' | Should Be $null
        }

        It 'Returns no logic for method/route that do not exist' {
            $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
            Find-PodeRoute -Method GET -Path '/' | Should Be $null
        }

        It 'Returns logic for method and exact route' {
            $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(@{ 'Logic'= { Write-Host 'Test' }; }); }; }; }
            $result = (Find-PodeRoute -Method GET -Path '/')

            $result | Should BeOfType System.Collections.Hashtable
            $result.Logic.ToString() | Should Be ({ Write-Host 'Test' }).ToString()
        }

        It 'Returns logic for method and exact route and protocol' {
            $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(
                @{ 'Logic'= { Write-Host 'Test' }; };
                @{ 'Logic'= { Write-Host 'Test' }; 'Protocol' = 'http' };
            ); }; }; }

            $result = (Find-PodeRoute -Method GET -Path '/' -Protocol 'http')

            $result | Should BeOfType System.Collections.Hashtable
            $result.Protocol | Should Be 'http'
            $result.Logic.ToString() | Should Be ({ Write-Host 'Test' }).ToString()
        }

        It 'Returns logic for method and exact route and endpoint' {
            $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(
                @{ 'Logic'= { Write-Host 'Test' }; };
                @{ 'Logic'= { Write-Host 'Test' }; 'Endpoint' = 'pode.foo.com' };
            ); }; }; }

            $result = (Find-PodeRoute -Method GET -Path '/' -Endpoint 'pode.foo.com')

            $result | Should BeOfType System.Collections.Hashtable
            $result.Endpoint | Should Be 'pode.foo.com'
            $result.Logic.ToString() | Should Be ({ Write-Host 'Test' }).ToString()
        }

        It 'Returns logic for method and exact route, endpoint and protocol' {
            $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(
                @{ 'Logic'= { Write-Host 'Test' }; };
                @{ 'Logic'= { Write-Host 'Test' }; 'Endpoint' = 'pode.foo.com' };
                @{ 'Logic'= { Write-Host 'Test' }; 'Endpoint' = 'pode.foo.com'; 'Protocol' = 'https' };
            ); }; }; }

            $result = (Find-PodeRoute -Method GET -Path '/' -Endpoint 'pode.foo.com' -Protocol 'https')

            $result | Should BeOfType System.Collections.Hashtable
            $result.Protocol | Should Be 'https'
            $result.Endpoint | Should Be 'pode.foo.com'
            $result.Logic.ToString() | Should Be ({ Write-Host 'Test' }).ToString()
        }

        It 'Returns logic for method and exact route and wildcard endpoint' {
            $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(
                @{ 'Logic'= { Write-Host 'Test' }; };
                @{ 'Logic'= { Write-Host 'Test' }; 'Endpoint' = '*:8080' };
            ); }; }; }

            $result = (Find-PodeRoute -Method GET -Path '/' -Endpoint 'localhost:8080')

            $result | Should BeOfType System.Collections.Hashtable
            $result.Endpoint | Should Be '*:8080'
            $result.Logic.ToString() | Should Be ({ Write-Host 'Test' }).ToString()
        }

        It 'Returns logic and middleware for method and exact route' {
            $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(@{ 'Logic'= { Write-Host 'Test' }; 'Middleware' = { Write-Host 'Middle' }; }); }; }; }
            $result = (Find-PodeRoute -Method GET -Path '/')

            $result | Should BeOfType System.Collections.Hashtable
            $result.Logic.ToString() | Should Be ({ Write-Host 'Test' }).ToString()
            $result.Middleware.ToString() | Should Be ({ Write-Host 'Middle' }).ToString()
        }

        It 'Returns logic for method and exact route under star' {
            $PodeContext.Server = @{ 'Routes' = @{ '*' = @{ '/' = @(@{ 'Logic'= { Write-Host 'Test' }; }); }; }; }
            $result = (Find-PodeRoute -Method * -Path '/')

            $result | Should BeOfType System.Collections.Hashtable
            $result.Logic.ToString() | Should Be ({ Write-Host 'Test' }).ToString()
        }

        It 'Returns logic and parameters for parameterised route' {
            $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/(?<userId>[^\/]+?)' = @(@{ 'Logic'= { Write-Host 'Test' }; }); }; }; }
            $result = (Find-PodeRoute -Method GET -Path '/123')

            $result | Should BeOfType System.Collections.Hashtable
            $result.Logic.ToString() | Should Be ({ Write-Host 'Test' }).ToString()
        }
    }
}

Describe 'Add-PodeStaticRoute' {
    It 'Adds basic static route' {
        Mock Test-PodePath { return $true }
        Mock New-PodePSDrive { return './assets' }

        $PodeContext.Server = @{ 'Routes' = @{ 'STATIC' = @{}; }; 'Root' = $pwd }
        Add-PodeStaticRoute -Path '/assets' -Source './assets'

        $route = $PodeContext.Server.Routes['static']
        $route | Should Not Be $null
        $route.ContainsKey('/assets[/]{0,1}(?<file>.*)') | Should Be $true
        $route['/assets[/]{0,1}(?<file>.*)'].Source | Should Be './assets'
    }

    It 'Throws error when adding static route for non-existing folder' {
        Mock Test-PodePath { return $false }
        $PodeContext.Server = @{ 'Routes' = @{ 'STATIC' = @{}; }; 'Root' = $pwd }
        { Add-PodeStaticRoute -Path '/assets' -Source './assets' } | Should Throw 'does not exist'
    }
}

Describe 'Remove-PodeRoute' {
    It 'Adds route with simple url, and then removes it' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }

        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'].Length | Should Be 1

        Remove-PodeRoute -Method Get -Path '/users'

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $false
    }

    It 'Adds two routes with simple url, and then removes one' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }

        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Get -Path '/users' -Protocol Http -ScriptBlock { Write-Host 'hello' }

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'].Length | Should Be 2

        Remove-PodeRoute -Method Get -Path '/users'

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'].Length | Should Be 1
    }
}

Describe 'Remove-PodeStaticRoute' {
    It 'Adds a static route, and then removes it' {
        Mock Test-PodePath { return $true }
        Mock New-PodePSDrive { return './assets' }

        $PodeContext.Server = @{ 'Routes' = @{ 'STATIC' = @{}; }; 'Root' = $pwd }
        Add-PodeStaticRoute -Path '/assets' -Source './assets'

        $routes = $PodeContext.Server.Routes['static']
        $routes | Should Not be $null
        $routes.ContainsKey('/assets[/]{0,1}(?<file>.*)') | Should Be $true
        $routes['/assets[/]{0,1}(?<file>.*)'].Source | Should Be './assets'

        Remove-PodeStaticRoute -Path '/assets'

        $routes = $PodeContext.Server.Routes['static']
        $routes | Should Not be $null
        $routes.ContainsKey('/assets[/]{0,1}(?<file>.*)') | Should Be $false
    }
}

Describe 'Clear-PodeRoutes' {
    It 'Adds routes for methods, and clears everything' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; 'POST' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello1' }
        Add-PodeRoute -Method POST -Path '/messages' -ScriptBlock { Write-Host 'hello2' }

        $routes = $PodeContext.Server.Routes['get']
        $routes.ContainsKey('/users') | Should Be $true

        $routes = $PodeContext.Server.Routes['post']
        $routes.ContainsKey('/messages') | Should Be $true

        Clear-PodeRoutes

        $routes = $PodeContext.Server.Routes['get']
        $routes.ContainsKey('/users') | Should Be $false

        $routes = $PodeContext.Server.Routes['post']
        $routes.ContainsKey('/messages') | Should Be $false
    }

    It 'Adds routes for methods, and clears one method' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; 'POST' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello1' }
        Add-PodeRoute -Method POST -Path '/messages' -ScriptBlock { Write-Host 'hello2' }

        $routes = $PodeContext.Server.Routes['get']
        $routes.ContainsKey('/users') | Should Be $true

        $routes = $PodeContext.Server.Routes['post']
        $routes.ContainsKey('/messages') | Should Be $true

        Clear-PodeRoutes -Method Get

        $routes = $PodeContext.Server.Routes['get']
        $routes.ContainsKey('/users') | Should Be $false

        $routes = $PodeContext.Server.Routes['post']
        $routes.ContainsKey('/messages') | Should Be $true
    }
}

Describe 'Clear-PodeStaticRoutes' {
    It 'Adds some static routes, and clears them all' {
        Mock Test-PodePath { return $true }
        Mock New-PodePSDrive { return './assets' }

        $PodeContext.Server = @{ 'Routes' = @{ 'STATIC' = @{}; }; 'Root' = $pwd }

        Add-PodeStaticRoute -Path '/assets' -Source './assets'
        Add-PodeStaticRoute -Path '/images' -Source './images'

        $routes = $PodeContext.Server.Routes['static']
        $routes.ContainsKey('/assets[/]{0,1}(?<file>.*)') | Should Be $true
        $routes.ContainsKey('/images[/]{0,1}(?<file>.*)') | Should Be $true

        Clear-PodeStaticRoutes

        $routes = $PodeContext.Server.Routes['static']
        $routes.ContainsKey('/assets[/]{0,1}(?<file>.*)') | Should Be $false
        $routes.ContainsKey('/images[/]{0,1}(?<file>.*)') | Should Be $false
    }
}

Describe 'Add-PodeRoute' {
    It 'Throws invalid method error for no method' {
        { Add-PodeRoute -Method 'MOO' -Path '/' -ScriptBlock {} } | Should Throw "Cannot validate argument on parameter 'Method'"
    }

    It 'Throws null route parameter error' {
        { Add-PodeRoute -Method GET -Path $null -ScriptBlock {} } | Should Throw 'it is an empty string'
    }

    It 'Throws empty route parameter error' {
        { Add-PodeRoute -Method GET -Path ([string]::Empty) -ScriptBlock {} } | Should Throw 'it is an empty string'
    }

    It 'Throws error when scriptblock and file path supplied' {
        { Add-PodeRoute -Method GET -Path '/' -ScriptBlock { write-host 'hi' } -FilePath './path' } | Should Throw 'parameter set cannot be resolved'
    }

    It 'Throws error when file path is a directory' {
        Mock Get-PodeRelativePath { return $Path }
        Mock Test-PodePath { return $true }
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{} } }
        { Add-PodeRoute -Method GET -Path '/' -FilePath './path' } | Should Throw 'cannot be a wildcard or a directory'
    }

    It 'Throws error when file path is a wildcard' {
        Mock Get-PodeRelativePath { return $Path }
        Mock Test-PodePath { return $true }
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{} } }
        { Add-PodeRoute -Method GET -Path '/' -FilePath './path/*' } | Should Throw 'cannot be a wildcard or a directory'
    }

    It 'Throws error because no scriptblock supplied' {
        { Add-PodeRoute -Method GET -Path '/' -ScriptBlock {} } | Should Throw "No logic passed"
    }

    It 'Throws error because only querystring has been given' {
        { Add-PodeRoute -Method GET -Path "?k=v" -ScriptBlock { write-host 'hi' } } | Should Throw "No path supplied"
    }

    It 'Throws error because route already exists' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(
            @{'Protocol' = ''; 'Endpoint' = ''}
        ); }; }; }

        { Add-PodeRoute -Method GET -Path '/' -ScriptBlock { write-host 'hi' } } | Should Throw 'already defined'
    }

    It 'Throws error because route and protocol already exists' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(
            @{'Protocol' = ''; 'Endpoint' = ''}
            @{'Protocol' = 'http'; 'Endpoint' = ''}
        ); }; }; }

        { Add-PodeRoute -Method GET -Path '/' -Protocol 'http' -ScriptBlock { write-host 'hi' } } | Should Throw 'already defined for'
    }

    It 'Throws error because route and endpoint already exists' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(
            @{'Protocol' = ''; 'Endpoint' = ''}
            @{'Protocol' = ''; 'Endpoint' = 'pode.foo.com:*'}
        ); }; }; }

        { Add-PodeRoute -Method GET -Path '/' -Endpoint 'pode.foo.com' -ScriptBlock { write-host 'hi' } } | Should Throw 'already defined for'
    }

    It 'Throws error because route, endpoint and protocol already exists' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{ '/' = @(
            @{'Protocol' = ''; 'Endpoint' = ''}
            @{'Protocol' = ''; 'Endpoint' = 'pode.foo.com:*'}
            @{'Protocol' = 'https'; 'Endpoint' = 'pode.foo.com:*'}
        ); }; }; }

        { Add-PodeRoute -Method GET -Path '/' -Protocol 'https' -Endpoint 'pode.foo.com' -ScriptBlock {} } | Should Throw 'already defined for'
    }

    It 'Throws error on GET route for endpoint name not existing' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        { Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello' } -EndpointName 'test' } | Should Throw 'does not exist'
    }

    It 'Adds route with simple url' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].ContentType | Should Be ([string]::Empty)
    }

    It 'Adds route with simple url and scriptblock from file path' {
        Mock Get-PodeRelativePath { return $Path }
        Mock Test-PodePath { return $true }
        Mock Use-PodeScript { return { Write-Host 'bye' } }

        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -FilePath './path/route.ps1'

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'bye' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].ContentType | Should Be ([string]::Empty)
    }

    Mock Test-PodePath { return $false }

    It 'Adds route with simple url with content type' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -ContentType 'application/json' -ScriptBlock { Write-Host 'hello' }

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].ContentType | Should Be 'application/json'
    }

    It 'Adds route with simple url with default content type' {
        $PodeContext.Server = @{
            'Routes' = @{ 'GET' = @{}; };
            'Web' = @{ 'ContentType' = @{
                'Default' = 'text/xml';
                'Routes' = @{};
            } };
        }

        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].ContentType | Should Be 'text/xml'
    }

    It 'Adds route with simple url with route pattern content type' {
        $PodeContext.Server = @{
            'Routes' = @{ 'GET' = @{}; };
            'Web' = @{ 'ContentType' = @{
                'Default' = 'text/xml';
                'Routes' = @{ '/users' = 'text/plain' };
            } };
        }

        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].ContentType | Should Be 'text/plain'
    }

    It 'Adds route with full endpoint' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello' } -Endpoint 'pode.foo.com:8080'

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].Endpoint | Should Be 'pode.foo.com:8080'
    }

    It 'Adds route with wildcard host endpoint' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello' } -Endpoint '8080'

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].Endpoint | Should Be '*:8080'
    }

    It 'Adds route with wildcard port endpoint' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello' } -Endpoint 'pode.foo.com'

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].Endpoint | Should Be 'pode.foo.com:*'
    }

    It 'Adds route with http protocol' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -ScriptBlock { Write-Host 'hello' } -Protocol 'http'

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null
        $routes.ContainsKey('/users') | Should Be $true
        $routes['/users'] | Should Not Be $null
        $routes['/users'].Length | Should Be 1
        $routes['/users'][0].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $routes['/users'][0].Middleware | Should Be $null
        $routes['/users'][0].Protocol | Should Be 'http'
    }

    It 'Adds route with middleware supplied as scriptblock and no logic' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -Middleware ({ Write-Host 'middle' }) -ScriptBlock {}

        $route = $PodeContext.Server.Routes['get']
        $route | Should Not be $null

        $route = $route['/users']
        $route | Should Not Be $null

        $route.Middleware.Logic.ToString() | Should Be ({ Write-Host 'middle' }).ToString()
        $route.Logic | Should Be ({}).ToString()
    }

    It 'Adds route with middleware supplied as hashtable with null logic' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        { Add-PodeRoute -Method GET -Path '/users' -Middleware (@{ 'Logic' = $null }) -ScriptBlock {} } | Should Throw 'no logic defined'
    }

    It 'Adds route with middleware supplied as hashtable with invalid type logic' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        { Add-PodeRoute -Method GET -Path '/users' -Middleware (@{ 'Logic' = 74 }) -ScriptBlock {} } | Should Throw 'invalid logic type'
    }

    It 'Adds route with invalid middleware type' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        { Add-PodeRoute -Method GET -Path '/users' -Middleware 74 -ScriptBlock {} } | Should Throw 'invalid type'
    }

    It 'Adds route with middleware supplied as hashtable and empty logic' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -Middleware (@{ 'Logic' = { Write-Host 'middle' }; 'Arguments' = 'test' }) -ScriptBlock {}

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null

        $routes = $routes['/users']
        $routes | Should Not Be $null
        $routes.Length | Should Be 1

        $routes[0].Logic.ToString() | Should Be ({}).ToString()
        $routes[0].Protocol | Should Be ''
        $routes[0].Endpoint | Should Be ''

        $routes[0].Middleware.Length | Should Be 1
        $routes[0].Middleware[0].Logic.ToString() | Should Be ({ Write-Host 'middle' }).ToString()
        $routes[0].Middleware[0].Arguments | Should Be 'test'
    }

    It 'Adds route with middleware supplied as hashtable and no logic' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -Middleware (@{ 'Logic' = { Write-Host 'middle' }; 'Arguments' = 'test' }) -ScriptBlock {}

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null

        $routes = $routes['/users']
        $routes | Should Not Be $null
        $routes.Length | Should Be 1

        $routes[0].Logic.ToString() | Should Be ({}).ToString()
        $routes[0].Protocol | Should Be ''
        $routes[0].Endpoint | Should Be ''

        $routes[0].Middleware.Length | Should Be 1
        $routes[0].Middleware[0].Logic.ToString() | Should Be ({ Write-Host 'middle' }).ToString()
        $routes[0].Middleware[0].Arguments | Should Be 'test'
    }

    It 'Adds route with middleware and logic supplied' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -Middleware { Write-Host 'middle' } -ScriptBlock { Write-Host 'logic' }

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null

        $routes = $routes['/users']
        $routes | Should Not Be $null
        $routes.Length | Should Be 1

        $routes[0].Logic.ToString() | Should Be ({ Write-Host 'logic' }).ToString()
        $routes[0].Protocol | Should Be ''
        $routes[0].Endpoint | Should Be ''

        $routes[0].Middleware.Length | Should Be 1
        $routes[0].Middleware[0].Logic.ToString() | Should Be ({ Write-Host 'middle' }).ToString()
    }

    It 'Adds route with array of middleware and no logic supplied' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }

        Add-PodeRoute -Method GET -Path '/users' -Middleware @(
            { Write-Host 'middle1' },
            { Write-Host 'middle2' }
            ) -ScriptBlock {}

        $routes = $PodeContext.Server.Routes['get']
        $routes | Should Not be $null

        $routes = $routes['/users']
        $routes | Should Not Be $null
        $routes.Length | Should Be 1

        $routes[0].Logic.ToString() | Should Be ({}).ToString()

        $routes[0].Middleware.Length | Should Be 2
        $routes[0].Middleware[0].Logic.ToString() | Should Be ({ Write-Host 'middle1' }).ToString()
        $routes[0].Middleware[1].Logic.ToString() | Should Be ({ Write-Host 'middle2' }).ToString()
    }

    It 'Adds route with array of middleware and logic supplied' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users' -Middleware @(
            { Write-Host 'middle1' },
            { Write-Host 'middle2' }
            ) -ScriptBlock { Write-Host 'logic' }

        $route = $PodeContext.Server.Routes['get']
        $route | Should Not be $null

        $route = $route['/users']
        $route | Should Not Be $null

        $route.Logic.ToString() | Should Be ({ Write-Host 'logic' }).ToString()
        $route.Middleware.Length | Should Be 2
        $route.Middleware[0].Logic.ToString() | Should Be ({ Write-Host 'middle1' }).ToString()
        $route.Middleware[1].Logic.ToString() | Should Be ({ Write-Host 'middle2' }).ToString()
    }

    It 'Adds route with simple url and querystring' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users?k=v' -ScriptBlock { Write-Host 'hello' }

        $route = $PodeContext.Server.Routes['get']
        $route | Should Not be $null
        $route.ContainsKey('/users') | Should Be $true
        $route['/users'] | Should Not Be $null
        $route['/users'].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $route['/users'].Middleware | Should Be $null
    }

    It 'Adds route with url parameters' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users/:userId' -ScriptBlock { Write-Host 'hello' }

        $route = $PodeContext.Server.Routes['get']
        $route | Should Not be $null
        $route.ContainsKey('/users/(?<userId>[^\/]+?)') | Should Be $true
        $route['/users/(?<userId>[^\/]+?)'] | Should Not Be $null
        $route['/users/(?<userId>[^\/]+?)'].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $route['/users/(?<userId>[^\/]+?)'].Middleware | Should Be $null
    }

    It 'Adds route with url parameters and querystring' {
        $PodeContext.Server = @{ 'Routes' = @{ 'GET' = @{}; }; }
        Add-PodeRoute -Method GET -Path '/users/:userId?k=v' -ScriptBlock { Write-Host 'hello' }

        $route = $PodeContext.Server.Routes['get']
        $route | Should Not be $null
        $route.ContainsKey('/users/(?<userId>[^\/]+?)') | Should Be $true
        $route['/users/(?<userId>[^\/]+?)'] | Should Not Be $null
        $route['/users/(?<userId>[^\/]+?)'].Logic.ToString() | Should Be ({ Write-Host 'hello' }).ToString()
        $route['/users/(?<userId>[^\/]+?)'].Middleware | Should Be $null
    }
}

Describe 'Convert-PodeFunctionVerbToHttpMethod' {
    It 'Returns POST for no Verb' {
        Convert-PodeFunctionVerbToHttpMethod -Verb ([string]::Empty) | Should Be 'POST'
    }

    It 'Returns POST' {
        Convert-PodeFunctionVerbToHttpMethod -Verb Invoke | Should Be 'POST'
    }

    It 'Returns GET' {
        Convert-PodeFunctionVerbToHttpMethod -Verb Find | Should Be 'GET'
    }

    It 'Returns PUT' {
        Convert-PodeFunctionVerbToHttpMethod -Verb Set | Should Be 'PUT'
    }

    It 'Returns PATCH' {
        Convert-PodeFunctionVerbToHttpMethod -Verb Edit | Should Be 'PATCH'
    }

    It 'Returns DELETE' {
        Convert-PodeFunctionVerbToHttpMethod -Verb Remove | Should Be 'DELETE'
    }
}

Describe 'ConvertTo-PodeRoute' {
    Mock Import-PodeModule {}
    Mock Write-Verbose {}
    Mock Add-PodeRoute {}
    Mock Write-PodeJsonResponse {}
    Mock Get-Module { return @{ ExportedCommands = @{ Keys = @('Some-ModuleCommand1', 'Some-ModuleCommand2') } } }

    It 'Throws error when module does not contain command' {
        { ConvertTo-PodeRoute -Module Example -Commands 'Get-ChildItem' } | Should Throw 'does not contain function'
    }

    It 'Throws error for no commands' {
        { ConvertTo-PodeRoute } | Should Throw 'No commands supplied to convert to Routes'
    }

    It 'Calls Add-PodeRoute twice for commands' {
        ConvertTo-PodeRoute -Commands @('Get-ChildItem', 'Invoke-Expression') -NoOpenApi
        Assert-MockCalled Add-PodeRoute -Times 2 -Scope It
    }

    It 'Calls Add-PodeRoute twice for module commands' {
        ConvertTo-PodeRoute -Module Example -NoOpenApi
        Assert-MockCalled Add-PodeRoute -Times 2 -Scope It
    }

    It 'Calls Add-PodeRoute once for module filtered commands' {
        ConvertTo-PodeRoute -Module Example -Commands 'Some-ModuleCommand1' -NoOpenApi
        Assert-MockCalled Add-PodeRoute -Times 1 -Scope It
    }
}

Describe 'Add-PodePage' {
    Mock Add-PodeRoute {}

    It 'Throws error for invalid Name' {
        { Add-PodePage -Name 'Rick+Morty' -ScriptBlock {} } | Should Throw 'should be a valid alphanumeric'
    }

    It 'Throws error for invalid ScriptBlock' {
        { Add-PodePage -Name 'RickMorty' -ScriptBlock {} } | Should Throw 'non-empty scriptblock is required'
    }

    It 'Throws error for invalid FilePath' {
        $PodeContext.Server = @{ 'Root' = $pwd }
        { Add-PodePage -Name 'RickMorty' -FilePath './fake/path' } | Should Throw 'the path does not exist'
    }

    It 'Call Add-PodeRoute once for ScriptBlock page' {
        Add-PodePage -Name 'Name' -ScriptBlock { Get-Service }
        Assert-MockCalled Add-PodeRoute -Times 1 -Scope It
    }

    It 'Call Add-PodeRoute once for FilePath page' {
        Mock Get-PodeRelativePath { return $Path }
        Add-PodePage -Name 'Name' -FilePath './fake/path'
        Assert-MockCalled Add-PodeRoute -Times 1 -Scope It
    }

    It 'Call Add-PodeRoute once for FilePath page' {
        Add-PodePage -Name 'Name' -View 'index'
        Assert-MockCalled Add-PodeRoute -Times 1 -Scope It
    }
}

Describe 'Update-PodeRouteSlashes' {
    Context 'Static' {
        It 'Update route slashes' {
            $input = '/route'
            Update-PodeRouteSlashes -Path $input -Static | Should Be '/route[/]{0,1}(?<file>.*)'
        }

        It 'Update route slashes, no slash' {
            $input = 'route'
            Update-PodeRouteSlashes -Path $input -Static | Should Be '/route[/]{0,1}(?<file>.*)'
        }

        It 'Update route slashes, ending with wildcard' {
            $input = '/route/*'
            Update-PodeRouteSlashes -Path $input -Static | Should Be '/route[/]{0,1}(?<file>.*)'
        }

        It 'Update route slashes, ending with wildcard, no slash' {
            $input = 'route/*'
            Update-PodeRouteSlashes -Path $input -Static | Should Be '/route[/]{0,1}(?<file>.*)'
        }

        It 'Update route slashes, with midpoint wildcard' {
            $input = '/route/*/ending'
            Update-PodeRouteSlashes -Path $input -Static | Should Be '/route/.*/ending[/]{0,1}(?<file>.*)'
        }

        It 'Update route slashes, with midpoint wildcard, no slash' {
            $input = 'route/*/ending'
            Update-PodeRouteSlashes -Path $input -Static | Should Be '/route/.*/ending[/]{0,1}(?<file>.*)'
        }

        It 'Update route slashes, with midpoint wildcard, ending with wildcard' {
            $input = '/route/*/ending/*'
            Update-PodeRouteSlashes -Path $input -Static | Should Be '/route/.*/ending[/]{0,1}(?<file>.*)'
        }

        It 'Update route slashes, with midpoint wildcard, ending with wildcard, no slash' {
            $input = 'route/*/ending/*'
            Update-PodeRouteSlashes -Path $input -Static | Should Be '/route/.*/ending[/]{0,1}(?<file>.*)'
        }
    }

    Context 'Non Static' {
        It 'Update route slashes' {
            $input = '/route'
            Update-PodeRouteSlashes -Path $input | Should Be '/route'
        }

        It 'Update route slashes, no slash' {
            $input = 'route'
            Update-PodeRouteSlashes -Path $input | Should Be '/route'
        }

        It 'Update route slashes, ending with wildcard' {
            $input = '/route/*'
            Update-PodeRouteSlashes -Path $input | Should Be '/route/.*'
        }

        It 'Update route slashes, ending with wildcard, no slash' {
            $input = 'route/*'
            Update-PodeRouteSlashes -Path $input | Should Be '/route/.*'
        }

        It 'Update route slashes, with midpoint wildcard' {
            $input = '/route/*/ending'
            Update-PodeRouteSlashes -Path $input | Should Be '/route/.*/ending'
        }

        It 'Update route slashes, with midpoint wildcard, no slash' {
            $input = 'route/*/ending'
            Update-PodeRouteSlashes -Path $input | Should Be '/route/.*/ending'
        }

        It 'Update route slashes, with midpoint wildcard, ending with wildcard' {
            $input = '/route/*/ending/*'
            Update-PodeRouteSlashes -Path $input | Should Be '/route/.*/ending/.*'
        }

        It 'Update route slashes, with midpoint wildcard, ending with wildcard, no slash' {
            $input = 'route/*/ending/*'
            Update-PodeRouteSlashes -Path $input | Should Be '/route/.*/ending/.*'
        }
    }
}

Describe 'Update-PodeRoutePlaceholders' {
    It 'Update route placeholders, basic' {
        $input = 'route'
        Update-PodeRoutePlaceholders -Path $input | Should Be 'route'
    }

    It 'Update route placeholders' {
        $input = ':route'
        Update-PodeRoutePlaceholders -Path $input | Should Be '(?<route>[^\/]+?)'
    }

    It 'Update route placeholders, double with no spacing' {
        $input = ':route:placeholder'
        Update-PodeRoutePlaceholders -Path $input | Should Be '(?<route>[^\/]+?)(?<placeholder>[^\/]+?)'
    }

    It 'Update route placeholders, double with double ::' {
        $input = '::route:placeholder'
        Update-PodeRoutePlaceholders -Path $input | Should Be ':(?<route>[^\/]+?)(?<placeholder>[^\/]+?)'
    }

    It 'Update route placeholders, double with slash' {
        $input = ':route/:placeholder'
        Update-PodeRoutePlaceholders -Path $input | Should Be '(?<route>[^\/]+?)/(?<placeholder>[^\/]+?)'
    }

    It 'Update route placeholders, no update' {
        $input = ': route'
        Update-PodeRoutePlaceholders -Path $input | Should Be ': route'
    }
}

Describe 'Split-PodeRouteQuery' {
    It 'Split route, no split' {
        $input = 'route'
        Split-PodeRouteQuery -Path $input | Should Be 'route'
    }

    It 'Split route, split' {
        $input = 'route?'
        Split-PodeRouteQuery -Path $input | Should Be 'route'
    }

    It 'Split route, split' {
        $input = 'route?split'
        Split-PodeRouteQuery -Path $input | Should Be 'route'
    }

    It 'Split route, split, first character' {
        $input = '?route'
        Split-PodeRouteQuery -Path $input | Should Be ''
    }
}

Describe 'Get-PodeRouteByUrl' {
    $routeBothSet = @{
        Protocol = 'HTTP'
        Endpoint = '/assets'
    }

    $routeEndpointSet = @{
        Protocol = ''
        Endpoint = '/assets'
    }

    $routeProtocolSet = @{
        Protocol = 'HTTP'
        Endpoint = ''
    }

    $routeNeitherSet = @{
        Protocol = ''
        Endpoint = ''
    }

    It 'Single route' {
        $Routes = @($routeBothSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeBothSet
    }

    It 'Single route, no protocol on route' {
        $Routes = @($routeEndpointSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeEndpointSet
    }

    It 'Single route, no endpoint on route' {
        $Routes = @($routeProtocolSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeProtocolSet
    }

    It 'Single route, no protcol or endpoint on route' {
        $Routes = @($routeNeitherSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeNeitherSet
    }

    It 'Single route, different casing, protcol' {
        $Routes = @($routeBothSet)

        $InputProtocol = 'HTTp'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeBothSet
    }

    It 'Single route, different casing, endpoint' {
        $Routes = @($routeBothSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/asSets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeBothSet
    }

    It 'Single route, different casing, protocol endpoint' {
        $Routes = @($routeBothSet)

        $InputProtocol = 'HTtP'
        $InputEndpoint = '/assEts'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeBothSet
    }

    It 'Single route, not passing protocol, no routes with no protocol' {
        $Routes = @($routeBothSet)

        $InputProtocol = ''
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Be $null
    }

    It 'Single route, not passing endpoint, no routes with no endpoint' {
        $Routes = @($routeBothSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = ''

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Be $null
    }

    It 'Single route, not passing protocol and endpoint, no routes with no protocol and endpoint' {
        $Routes = @($routeBothSet)

        $InputProtocol = ''
        $InputEndpoint = ''

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Be $null
    }

    It 'Single route, not passing protocol, route with no protocol and endpoint' {
        $Routes = @($routeNeitherSet)

        $InputProtocol = ''
        $InputEndpoint = '/assests'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeNeitherSet
    }

    It 'Single route, not passing endpoint, route with no protocol and endpoint' {
        $Routes = @($routeNeitherSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = ''

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeNeitherSet
    }

    It 'Single route, not passing protocol and endpoint, route with no protocol and endpoint' {
        $Routes = @($routeNeitherSet)

        $InputProtocol = ''
        $InputEndpoint = ''

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeNeitherSet
    }

    It 'No routes' {
        $Routes = @()

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Be $null
    }

    It 'Two routes, sorting' {
        $Routes = @($routeEndpointSet, $routeBothSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeBothSet
    }

    It 'Two routes, sorting' {
        $Routes = @($routeBothSet, $routeProtocolSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeBothSet
    }

    It 'Two routes, sorting' {
        $Routes = @($routeBothSet, $routeNeitherSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeBothSet
    }

    It 'Many routes, sorting' {
        $Routes = @($routeProtocolSet, $routeEndpointSet, $routeBothSet, $routeNeitherSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeBothSet
    }

    It 'Two routes, not passing protocol, route with no protocol and endpoint' {
        $Routes = @($routeBothSet, $routeNeitherSet)

        $InputProtocol = ''
        $InputEndpoint = '/assets'

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeNeitherSet
    }

    It 'Two routes, not passing endpoint, route with no protocol and endpoint' {
        $Routes = @($routeBothSet, $routeNeitherSet)

        $InputProtocol = 'HTTP'
        $InputEndpoint = ''

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeNeitherSet
    }

    It 'Two routes, not passing procotol and endpoint, route with no protocol and endpoint' {
        $Routes = @($routeBothSet, $routeNeitherSet)

        $InputProtocol = ''
        $InputEndpoint = ''

        $Result = Get-PodeRouteByUrl -Routes $Routes -Protocol $InputProtocol -Endpoint $InputEndpoint

        $Result | Should Not Be $null
        $Result | Should Be $routeNeitherSet
    }
}

Describe 'Get-PodeRoute' {
    Mock Test-PodeIPAddress { return $true }
    Mock Test-IsAdminUser { return $true }

    It 'Returns both routes whe nothing supplied' {
        $PodeContext.Server = @{ Routes = @{ GET = @{}; POST = @{}; }; }
        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Get -Path '/about' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Post -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = Get-PodeRoute
        $routes.Length | Should Be 3
    }

    It 'Returns both routes for GET method' {
        $PodeContext.Server = @{ Routes = @{ GET = @{}; POST = @{}; }; }
        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Get -Path '/about' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Post -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = Get-PodeRoute -Method Get
        $routes.Length | Should Be 2
    }

    It 'Returns one route for POST method' {
        $PodeContext.Server = @{ Routes = @{ GET = @{}; POST = @{}; }; }
        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Get -Path '/about' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Post -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = Get-PodeRoute -Method Post
        $routes.Length | Should Be 1
    }

    It 'Returns both routes for users path' {
        $PodeContext.Server = @{ Routes = @{ GET = @{}; POST = @{}; }; }
        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Get -Path '/about' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Post -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = Get-PodeRoute -Path '/users'
        $routes.Length | Should Be 2
    }

    It 'Returns one route for users path and GET method' {
        $PodeContext.Server = @{ Routes = @{ GET = @{}; POST = @{}; }; }
        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Get -Path '/about' -ScriptBlock { Write-Host 'hello' }
        Add-PodeRoute -Method Post -Path '/users' -ScriptBlock { Write-Host 'hello' }

        $routes = Get-PodeRoute -Method Get -Path '/users'
        $routes.Length | Should Be 1
    }

    It 'Returns one route for users path and endpoint' {
        $PodeContext.Server = @{ Routes = @{ GET = @{}; POST = @{}; }; Endpoints = @(); Type = $null }

        Add-PodeEndpoint -Address '127.0.0.1' -Port 8080 -Protocol Http
        Add-PodeEndpoint -Address '127.0.0.1' -Port 8081 -Protocol Http

        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' } -Endpoint 127.0.0.1:8080
        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' } -Endpoint 127.0.0.1:8081

        $routes = @(Get-PodeRoute -Method Get -Path '/users' -Endpoint 127.0.0.1:8080)
        $routes.Length | Should Be 1
        $routes[0].Endpoint | Should Be '127.0.0.1:8080'
    }

    It 'Returns one route for users path and endpoint name user' {
        $PodeContext.Server = @{ Routes = @{ GET = @{}; POST = @{}; }; Endpoints = @(); Type = $null }

        Add-PodeEndpoint -Address '127.0.0.1' -Port 8080 -Protocol Http -Name user
        Add-PodeEndpoint -Address '127.0.0.1' -Port 8081 -Protocol Http -Name admin

        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' } -EndpointName user
        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' } -EndpointName admin

        $routes = @(Get-PodeRoute -Method Get -Path '/users' -EndpointName user)
        $routes.Length | Should Be 1
        $routes[0].EndpointName | Should Be 'user'
        $routes[0].Endpoint | Should Be '127.0.0.1:8080'
    }

    It 'Returns both routes for users path and endpoint names' {
        $PodeContext.Server = @{ Routes = @{ GET = @{}; POST = @{}; }; Endpoints = @(); Type = $null }

        Add-PodeEndpoint -Address '127.0.0.1' -Port 8080 -Protocol Http -Name user
        Add-PodeEndpoint -Address '127.0.0.1' -Port 8081 -Protocol Http -Name admin

        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' } -EndpointName user
        Add-PodeRoute -Method Get -Path '/users' -ScriptBlock { Write-Host 'hello' } -EndpointName admin

        $routes = @(Get-PodeRoute -Method Get -Path '/users' -EndpointName user, admin)
        $routes.Length | Should Be 2
    }
}

Describe 'Get-PodeStaticRoute' {
    Mock Test-PodePath { return $true }
    Mock New-PodePSDrive { return './assets' }

    It 'Returns all static routes' {
        $PodeContext.Server = @{ Routes = @{ STATIC = @{}; }; Root = $pwd }
        Add-PodeStaticRoute -Path '/assets' -Source './assets'
        Add-PodeStaticRoute -Path '/images' -Source './images'

        $routes = Get-PodeStaticRoute
        $routes.Length | Should Be 2
    }

    It 'Returns one static route' {
        $PodeContext.Server = @{ Routes = @{ STATIC = @{}; }; Root = $pwd }
        Add-PodeStaticRoute -Path '/assets' -Source './assets'
        Add-PodeStaticRoute -Path '/images' -Source './images'

        $routes = Get-PodeStaticRoute -Path '/images'
        $routes.Length | Should Be 1
    }




    

    It 'Returns one static route for endpoint' {
        $PodeContext.Server = @{ Routes = @{ STATIC = @{}; }; Root = $pwd; Endpoints = @(); Type = $null }

        Add-PodeEndpoint -Address '127.0.0.1' -Port 8080 -Protocol Http
        Add-PodeEndpoint -Address '127.0.0.1' -Port 8081 -Protocol Http

        Add-PodeStaticRoute -Path '/images' -Source './images' -Endpoint 127.0.0.1:8080
        Add-PodeStaticRoute -Path '/images' -Source './images' -Endpoint 127.0.0.1:8081

        $routes = @(Get-PodeStaticRoute -Path '/images' -Endpoint 127.0.0.1:8080)
        $routes.Length | Should Be 1
        $routes[0].Endpoint | Should Be '127.0.0.1:8080'
    }

    It 'Returns one static route for endpoint name user' {
        $PodeContext.Server = @{ Routes = @{ STATIC = @{}; }; Root = $pwd; Endpoints = @(); Type = $null }

        Add-PodeEndpoint -Address '127.0.0.1' -Port 8080 -Protocol Http -Name user
        Add-PodeEndpoint -Address '127.0.0.1' -Port 8081 -Protocol Http -Name admin

        Add-PodeStaticRoute -Path '/images' -Source './images' -EndpointName user
        Add-PodeStaticRoute -Path '/images' -Source './images' -EndpointName admin

        $routes = @(Get-PodeStaticRoute -Path '/images' -EndpointName user)
        $routes.Length | Should Be 1
        $routes[0].EndpointName | Should Be 'user'
        $routes[0].Endpoint | Should Be '127.0.0.1:8080'
    }

    It 'Returns both routes for users path and endpoint names' {
        $PodeContext.Server = @{ Routes = @{ STATIC = @{}; }; Root = $pwd; Endpoints = @(); Type = $null }

        Add-PodeEndpoint -Address '127.0.0.1' -Port 8080 -Protocol Http -Name user
        Add-PodeEndpoint -Address '127.0.0.1' -Port 8081 -Protocol Http -Name admin

        Add-PodeStaticRoute -Path '/images' -Source './images' -EndpointName user
        Add-PodeStaticRoute -Path '/images' -Source './images' -EndpointName admin

        $routes = @(Get-PodeStaticRoute -Path '/images' -EndpointName user, admin)
        $routes.Length | Should Be 2
    }
}